@@ -14,24 +14,41 @@
  *                                                                         *
  ***************************************************************************/
 
+/**
+    \mainpage libftdi API documentation
+
+    Library to talk to FTDI chips. You find the latest versions of libftdi at
+    http://www.intra2net.com/de/produkte/opensource/ftdi/
+
+    The library is easy to use. Have a look at this short example:
+    \include simple.c
+
+    More examples can be found in the "examples" directory.
+*/
+/** \addtogroup libftdi */
+/* @{ */
+
 #include <usb.h>
 #include <string.h>
+#include <errno.h>
 
 #include "ftdi.h"
 
 #define ftdi_error_return(code, str) do {  \
         ftdi->error_str = str;             \
         return code;                       \
-   } while(0);                 
+   } while(0);
 
 
-/* ftdi_init
+/**
+    Initializes a ftdi_context.
 
-  Initializes a ftdi_context.
+    \param ftdi pointer to ftdi_context
 
-  Return codes:
-   0: All fine
-  -1: Couldn't allocate read buffer
+    \retval  0: all fine
+    \retval -1: couldn't allocate read buffer
+
+    \remark This should be called before all functions
 */
 int ftdi_init(struct ftdi_context *ftdi)
 {
@@ -60,13 +77,14 @@ int ftdi_init(struct ftdi_context *ftdi)
     return ftdi_read_data_set_chunksize(ftdi, 4096);
 }
 
-/* ftdi_set_interface
-   
-   Call after ftdi_init
-   
-   Open selected channels on a chip, otherwise use first channel
-    0: all fine
-   -1: unknown interface
+/**
+    Open selected channels on a chip, otherwise use first channel.
+
+    \param ftdi pointer to ftdi_context
+    \param interface Interface to use for FT2232C chips.
+
+    \retval  0: all fine
+    \retval -1: unknown interface
 */
 int ftdi_set_interface(struct ftdi_context *ftdi, enum ftdi_interface interface)
 {
@@ -87,9 +105,10 @@ int ftdi_set_interface(struct ftdi_context *ftdi, enum ftdi_interface interface)
     return 0;
 }
 
-/* ftdi_deinit
+/**
+    Deinitializes a ftdi_context.
 
-   Deinitializes a ftdi_context.
+    \param ftdi pointer to ftdi_context
 */
 void ftdi_deinit(struct ftdi_context *ftdi)
 {
@@ -99,9 +118,11 @@ void ftdi_deinit(struct ftdi_context *ftdi)
     }
 }
 
-/* ftdi_set_usbdev
- 
-   Use an already open device.
+/**
+    Use an already open libusb device.
+
+    \param ftdi pointer to ftdi_context
+    \param usb libusb usb_dev_handle to use
 */
 void ftdi_set_usbdev (struct ftdi_context *ftdi, usb_dev_handle *usb)
 {
@@ -109,24 +130,27 @@ void ftdi_set_usbdev (struct ftdi_context *ftdi, usb_dev_handle *usb)
 }
 
 
-/* ftdi_usb_find_all
- 
-   Finds all ftdi devices on the usb bus. Creates a new ftdi_device_list which
-   needs to be deallocated by ftdi_list_free after use.
+/**
+    Finds all ftdi devices on the usb bus. Creates a new ftdi_device_list which
+    needs to be deallocated by ftdi_list_free() after use.
 
-   Return codes:
-    >0: number of devices found
-    -1: usb_find_busses() failed
-    -2: usb_find_devices() failed
-    -3: out of memory
+    \param ftdi pointer to ftdi_context
+    \param devlist Pointer where to store list of found devices
+    \param vendor Vendor ID to search for
+    \param product Product ID to search for
+
+    \retval >0: number of devices found
+    \retval -1: usb_find_busses() failed
+    \retval -2: usb_find_devices() failed
+    \retval -3: out of memory
 */
-int ftdi_usb_find_all(struct ftdi_context *ftdi, struct ftdi_device_list **devlist, int vendor, int product) 
+int ftdi_usb_find_all(struct ftdi_context *ftdi, struct ftdi_device_list **devlist, int vendor, int product)
 {
     struct ftdi_device_list **curdev;
     struct usb_bus *bus;
     struct usb_device *dev;
     int count = 0;
-    
+
     usb_init();
     if (usb_find_busses() < 0)
         ftdi_error_return(-1, "usb_find_busses() failed");
@@ -134,6 +158,7 @@ int ftdi_usb_find_all(struct ftdi_context *ftdi, struct ftdi_device_list **devli
         ftdi_error_return(-2, "usb_find_devices() failed");
 
     curdev = devlist;
+    *curdev = NULL;
     for (bus = usb_busses; bus; bus = bus->next) {
         for (dev = bus->devices; dev; dev = dev->next) {
             if (dev->descriptor.idVendor == vendor
@@ -142,7 +167,7 @@ int ftdi_usb_find_all(struct ftdi_context *ftdi, struct ftdi_device_list **devli
                 *curdev = (struct ftdi_device_list*)malloc(sizeof(struct ftdi_device_list));
                 if (!*curdev)
                     ftdi_error_return(-3, "out of memory");
-                
+
                 (*curdev)->next = NULL;
                 (*curdev)->dev = dev;
 
@@ -151,44 +176,122 @@ int ftdi_usb_find_all(struct ftdi_context *ftdi, struct ftdi_device_list **devli
             }
         }
     }
-    
+
     return count;
 }
 
-/* ftdi_list_free
+/**
+    Frees a usb device list.
 
-   Frees a created device list.
+    \param devlist USB device list created by ftdi_usb_find_all()
 */
-void ftdi_list_free(struct ftdi_device_list **devlist) 
+void ftdi_list_free(struct ftdi_device_list **devlist)
 {
-    struct ftdi_device_list **curdev;
-    for (; *devlist == NULL; devlist = curdev) {
-        curdev = &(*devlist)->next;
-        free(*devlist);
+    struct ftdi_device_list *curdev, *next;
+
+    for (curdev = *devlist; curdev != NULL;) {
+        next = curdev->next;
+        free(curdev);
+        curdev = next;
     }
 
-    devlist = NULL;
+    *devlist = NULL;
 }
 
-/* ftdi_usb_open_dev 
+/**
+    Return device ID strings from the usb device.
 
-   Opens a ftdi device given by a usb_device.
-   
-   Return codes:
-     0: all fine
-    -4: unable to open device
-    -5: unable to claim device
-    -6: reset failed
-    -7: set baudrate failed
+    The parameters manufacturer, description and serial may be NULL
+    or pointer to buffers to store the fetched strings.
+
+    \note Use this function only in combination with ftdi_usb_find_all()
+          as it closes the internal "usb_dev" after use.
+
+    \param ftdi pointer to ftdi_context
+    \param dev libusb usb_dev to use
+    \param manufacturer Store manufacturer string here if not NULL
+    \param mnf_len Buffer size of manufacturer string
+    \param description Store product description string here if not NULL
+    \param desc_len Buffer size of product description string
+    \param serial Store serial string here if not NULL
+    \param serial_len Buffer size of serial string
+
+    \retval   0: all fine
+    \retval  -1: wrong arguments
+    \retval  -4: unable to open device
+    \retval  -7: get product manufacturer failed
+    \retval  -8: get product description failed
+    \retval  -9: get serial number failed
+    \retval -10: unable to close device
+*/
+int ftdi_usb_get_strings(struct ftdi_context * ftdi, struct usb_device * dev,
+        char * manufacturer, int mnf_len, char * description, int desc_len, char * serial, int serial_len)
+{
+    if ((ftdi==NULL) || (dev==NULL))
+        return -1;
+
+    if (!(ftdi->usb_dev = usb_open(dev)))
+        ftdi_error_return(-4, usb_strerror());
+
+    if (manufacturer != NULL) {
+        if (usb_get_string_simple(ftdi->usb_dev, dev->descriptor.iManufacturer, manufacturer, mnf_len) <= 0) {
+            usb_close (ftdi->usb_dev);
+            ftdi_error_return(-7, usb_strerror());
+        }
+    }
+
+    if (description != NULL) {
+        if (usb_get_string_simple(ftdi->usb_dev, dev->descriptor.iProduct, description, desc_len) <= 0) {
+            usb_close (ftdi->usb_dev);
+            ftdi_error_return(-8, usb_strerror());
+        }
+    }
+
+    if (serial != NULL) {
+        if (usb_get_string_simple(ftdi->usb_dev, dev->descriptor.iSerialNumber, serial, serial_len) <= 0) {
+            usb_close (ftdi->usb_dev);
+            ftdi_error_return(-9, usb_strerror());
+        }
+    }
+
+    if (usb_close (ftdi->usb_dev) != 0)
+        ftdi_error_return(-10, usb_strerror());
+
+    return 0;
+}
+
+/**
+    Opens a ftdi device given by a usb_device.
+
+    \param ftdi pointer to ftdi_context
+    \param dev libusb usb_dev to use
+
+    \retval  0: all fine
+    \retval -4: unable to open device
+    \retval -5: unable to claim device
+    \retval -6: reset failed
+    \retval -7: set baudrate failed
 */
 int ftdi_usb_open_dev(struct ftdi_context *ftdi, struct usb_device *dev)
 {
+    int detach_errno = 0;
     if (!(ftdi->usb_dev = usb_open(dev)))
         ftdi_error_return(-4, "usb_open() failed");
-    
+
+#ifdef LIBUSB_HAS_GET_DRIVER_NP
+    // Try to detach ftdi_sio kernel module
+    // Returns ENODATA if driver is not loaded
+    if (usb_detach_kernel_driver_np(ftdi->usb_dev, ftdi->interface) != 0 && errno != ENODATA)
+        detach_errno = errno;
+#endif
+
     if (usb_claim_interface(ftdi->usb_dev, ftdi->interface) != 0) {
         usb_close (ftdi->usb_dev);
-        ftdi_error_return(-5, "unable to claim usb device. Make sure ftdi_sio is unloaded!");
+        if (detach_errno == EPERM) {
+            ftdi_error_return(-8, "inappropriate permissions on device!");
+        } else {
+            ftdi_error_return(-5, "unable to claim usb device. Make sure ftdi_sio is unloaded!");
+        }
     }
 
     if (ftdi_usb_reset (ftdi) != 0) {
@@ -217,35 +320,41 @@ int ftdi_usb_open_dev(struct ftdi_context *ftdi, struct usb_device *dev)
     ftdi_error_return(0, "all fine");
 }
 
-/* ftdi_usb_open
-   
-   Opens the first device with a given vendor and product ids.
-   
-   Return codes:
-   See ftdi_usb_open_desc()
-*/  
+/**
+    Opens the first device with a given vendor and product ids.
+
+    \param ftdi pointer to ftdi_context
+    \param vendor Vendor ID
+    \param product Product ID
+
+    \retval same as ftdi_usb_open_desc()
+*/
 int ftdi_usb_open(struct ftdi_context *ftdi, int vendor, int product)
 {
     return ftdi_usb_open_desc(ftdi, vendor, product, NULL, NULL);
 }
 
-/* ftdi_usb_open_desc
+/**
+    Opens the first device with a given, vendor id, product id,
+    description and serial.
 
-   Opens the first device with a given, vendor id, product id,
-   description and serial.
-   
-   Return codes:
-     0: all fine
-    -1: usb_find_busses() failed
-    -2: usb_find_devices() failed
-    -3: usb device not found
-    -4: unable to open device
-    -5: unable to claim device
-    -6: reset failed
-    -7: set baudrate failed
-    -8: get product description failed
-    -9: get serial number failed
-    -10: unable to close device
+    \param ftdi pointer to ftdi_context
+    \param vendor Vendor ID
+    \param product Product ID
+    \param description Description to search for. Use NULL if not needed.
+    \param serial Serial to search for. Use NULL if not needed.
+
+    \retval  0: all fine
+    \retval -1: usb_find_busses() failed
+    \retval -2: usb_find_devices() failed
+    \retval -3: usb device not found
+    \retval -4: unable to open device
+    \retval -5: unable to claim device
+    \retval -6: reset failed
+    \retval -7: set baudrate failed
+    \retval -8: get product description failed
+    \retval -9: get serial number failed
+    \retval -10: unable to close device
 */
 int ftdi_usb_open_desc(struct ftdi_context *ftdi, int vendor, int product,
                        const char* description, const char* serial)
@@ -293,7 +402,7 @@ int ftdi_usb_open_desc(struct ftdi_context *ftdi, int vendor, int product,
 
                 if (usb_close (ftdi->usb_dev) != 0)
                     ftdi_error_return(-10, "unable to close device");
-                
+
                 return ftdi_usb_open_dev(ftdi, dev);
             }
         }
@@ -303,13 +412,13 @@ int ftdi_usb_open_desc(struct ftdi_context *ftdi, int vendor, int product,
     ftdi_error_return(-3, "device not found");
 }
 
-/* ftdi_usb_reset
+/**
+    Resets the ftdi device.
 
-   Resets the ftdi device.
-   
-   Return codes:
-     0: all fine
-    -1: FTDI reset failed
+    \param ftdi pointer to ftdi_context
+
+    \retval  0: all fine
+    \retval -1: FTDI reset failed
 */
 int ftdi_usb_reset(struct ftdi_context *ftdi)
 {
@@ -323,14 +432,14 @@ int ftdi_usb_reset(struct ftdi_context *ftdi)
     return 0;
 }
 
-/* ftdi_usb_purge_buffers
+/**
+    Clears the buffers on the chip.
 
-   Cleans the buffers of the ftdi device.
-   
-   Return codes:
-     0: all fine
-    -1: write buffer purge failed
-    -2: read buffer purge failed
+    \param ftdi pointer to ftdi_context
+
+    \retval  0: all fine
+    \retval -1: write buffer purge failed
+    \retval -2: read buffer purge failed
 */
 int ftdi_usb_purge_buffers(struct ftdi_context *ftdi)
 {
@@ -347,14 +456,14 @@ int ftdi_usb_purge_buffers(struct ftdi_context *ftdi)
     return 0;
 }
 
-/* ftdi_usb_close
-   
-   Closes the ftdi device.
-   
-   Return codes:
-     0: all fine
-    -1: usb_release failed
-    -2: usb_close failed
+/**
+    Closes the ftdi device. Call ftdi_deinit() if you're cleaning up.
+
+    \param ftdi pointer to ftdi_context
+
+    \retval  0: all fine
+    \retval -1: usb_release failed
+    \retval -2: usb_close failed
 */
 int ftdi_usb_close(struct ftdi_context *ftdi)
 {
@@ -369,10 +478,10 @@ int ftdi_usb_close(struct ftdi_context *ftdi)
     return rtn;
 }
 
-
 /*
     ftdi_convert_baudrate returns nearest supported baud rate to that requested.
     Function is only used internally
+    \internal
 */
 static int ftdi_convert_baudrate(int baudrate, struct ftdi_context *ftdi,
                                  unsigned short *value, unsigned short *index)
@@ -471,15 +580,15 @@ static int ftdi_convert_baudrate(int baudrate, struct ftdi_context *ftdi,
     return best_baud;
 }
 
-/*
-    ftdi_set_baudrate
-    
-    Sets the chip baudrate
-    
-    Return codes:
-     0: all fine
-    -1: invalid baudrate
-    -2: setting baudrate failed
+/**
+    Sets the chip baud rate
+
+    \param ftdi pointer to ftdi_context
+    \param baudrate baud rate to set
+
+    \retval  0: all fine
+    \retval -1: invalid baudrate
+    \retval -2: setting baudrate failed
 */
 int ftdi_set_baudrate(struct ftdi_context *ftdi, int baudrate)
 {
@@ -508,17 +617,19 @@ int ftdi_set_baudrate(struct ftdi_context *ftdi, int baudrate)
     return 0;
 }
 
-/*
-    ftdi_set_line_property
+/**
+    Set (RS232) line characteristics by Alain Abbas
 
-    set (RS232) line characteristics by Alain Abbas
-    
-    Return codes:
-     0: all fine
-    -1: Setting line property failed
+    \param ftdi pointer to ftdi_context
+    \param bits Number of bits
+    \param sbit Number of stop bits
+    \param parity Parity mode
+
+    \retval  0: all fine
+    \retval -1: Setting line property failed
 */
 int ftdi_set_line_property(struct ftdi_context *ftdi, enum ftdi_bits_type bits,
-                    enum ftdi_stopbits_type sbit, enum ftdi_parity_type parity)
+                           enum ftdi_stopbits_type sbit, enum ftdi_parity_type parity)
 {
     unsigned short value = bits;
 
@@ -539,7 +650,7 @@ int ftdi_set_line_property(struct ftdi_context *ftdi, enum ftdi_bits_type bits,
         value |= (0x04 << 8);
         break;
     }
-    
+
     switch(sbit) {
     case STOP_BIT_1:
         value |= (0x00 << 11);
@@ -551,13 +662,23 @@ int ftdi_set_line_property(struct ftdi_context *ftdi, enum ftdi_bits_type bits,
         value |= (0x02 << 11);
         break;
     }
-    
+
     if (usb_control_msg(ftdi->usb_dev, 0x40, 0x04, value, ftdi->index, NULL, 0, ftdi->usb_write_timeout) != 0)
         ftdi_error_return (-1, "Setting new line property failed");
-    
+
     return 0;
 }
 
+/**
+    Writes data in chunks (see ftdi_write_data_set_chunksize()) to the chip
+
+    \param ftdi pointer to ftdi_context
+    \param buf Buffer with the data
+    \param size Size of the buffer
+
+    \retval <0: error code from usb_bulk_write()
+    \retval >0: number of bytes written
+*/
 int ftdi_write_data(struct ftdi_context *ftdi, unsigned char *buf, int size)
 {
     int ret;
@@ -581,21 +702,51 @@ int ftdi_write_data(struct ftdi_context *ftdi, unsigned char *buf, int size)
     return total_written;
 }
 
+/**
+    Configure write buffer chunk size.
+    Default is 4096.
 
+    \param ftdi pointer to ftdi_context
+    \param chunksize Chunk size
+
+    \retval 0: all fine
+*/
 int ftdi_write_data_set_chunksize(struct ftdi_context *ftdi, unsigned int chunksize)
 {
     ftdi->writebuffer_chunksize = chunksize;
     return 0;
 }
 
+/**
+    Get write buffer chunk size.
 
+    \param ftdi pointer to ftdi_context
+    \param chunksize Pointer to store chunk size in
+
+    \retval 0: all fine
+*/
 int ftdi_write_data_get_chunksize(struct ftdi_context *ftdi, unsigned int *chunksize)
 {
     *chunksize = ftdi->writebuffer_chunksize;
     return 0;
 }
 
+/**
+    Reads data in chunks (see ftdi_read_data_set_chunksize()) from the chip.
 
+    Automatically strips the two modem status bytes transfered during every read.
+
+    \param ftdi pointer to ftdi_context
+    \param buf Buffer to store data in
+    \param size Size of the buffer
+
+    \retval <0: error code from usb_bulk_read()
+    \retval  0: no data was available
+    \retval >0: number of bytes read
+
+    \remark This function is not useful in bitbang mode.
+            Use ftdi_read_pins() to get the current state of the pins.
+*/
 int ftdi_read_data(struct ftdi_context *ftdi, unsigned char *buf, int size)
 {
     int offset = 0, ret = 1, i, num_of_chunks, chunk_remains;
@@ -687,7 +838,17 @@ int ftdi_read_data(struct ftdi_context *ftdi, unsigned char *buf, int size)
     return -127;
 }
 
+/**
+    Configure read buffer chunk size.
+    Default is 4096.
 
+    Automatically reallocates the buffer.
+
+    \param ftdi pointer to ftdi_context
+    \param chunksize Chunk size
+
+    \retval 0: all fine
+*/
 int ftdi_read_data_set_chunksize(struct ftdi_context *ftdi, unsigned int chunksize)
 {
     unsigned char *new_buf;
@@ -705,7 +866,14 @@ int ftdi_read_data_set_chunksize(struct ftdi_context *ftdi, unsigned int chunksi
     return 0;
 }
 
+/**
+    Get read buffer chunk size.
 
+    \param ftdi pointer to ftdi_context
+    \param chunksize Pointer to store chunk size in
+
+    \retval 0: all fine
+*/
 int ftdi_read_data_get_chunksize(struct ftdi_context *ftdi, unsigned int *chunksize)
 {
     *chunksize = ftdi->readbuffer_chunksize;
@@ -713,7 +881,18 @@ int ftdi_read_data_get_chunksize(struct ftdi_context *ftdi, unsigned int *chunks
 }
 
 
+/**
+    Enable bitbang mode.
 
+    For advanced bitbang modes of the FT2232C chip use ftdi_set_bitmode().
+
+    \param ftdi pointer to ftdi_context
+    \param bitmask Bitmask to configure lines.
+           HIGH/ON value configures a line as output.
+
+    \retval  0: all fine
+    \retval -1: can't enable bitbang mode
+*/
 int ftdi_enable_bitbang(struct ftdi_context *ftdi, unsigned char bitmask)
 {
     unsigned short usb_val;
@@ -729,7 +908,14 @@ int ftdi_enable_bitbang(struct ftdi_context *ftdi, unsigned char bitmask)
     return 0;
 }
 
+/**
+    Disable bitbang mode.
 
+    \param ftdi pointer to ftdi_context
+
+    \retval  0: all fine
+    \retval -1: can't disable bitbang mode
+*/
 int ftdi_disable_bitbang(struct ftdi_context *ftdi)
 {
     if (usb_control_msg(ftdi->usb_dev, 0x40, 0x0B, 0, ftdi->index, NULL, 0, ftdi->usb_write_timeout) != 0)
@@ -739,7 +925,17 @@ int ftdi_disable_bitbang(struct ftdi_context *ftdi)
     return 0;
 }
 
+/**
+    Enable advanced bitbang mode for FT2232C chips.
 
+    \param ftdi pointer to ftdi_context
+    \param bitmask Bitmask to configure lines.
+           HIGH/ON value configures a line as output.
+    \param mode Bitbang mode: 1 for normal mode, 2 for SPI mode
+
+    \retval  0: all fine
+    \retval -1: can't enable bitbang mode
+*/
 int ftdi_set_bitmode(struct ftdi_context *ftdi, unsigned char bitmask, unsigned char mode)
 {
     unsigned short usb_val;
@@ -754,17 +950,37 @@ int ftdi_set_bitmode(struct ftdi_context *ftdi, unsigned char bitmask, unsigned
     return 0;
 }
 
+/**
+    Directly read pin state. Useful for bitbang mode.
+
+    \param ftdi pointer to ftdi_context
+    \param pins Pointer to store pins into
+
+    \retval  0: all fine
+    \retval -1: read pins failed
+*/
 int ftdi_read_pins(struct ftdi_context *ftdi, unsigned char *pins)
 {
-    unsigned short usb_val;
-    if (usb_control_msg(ftdi->usb_dev, 0xC0, 0x0C, 0, ftdi->index, (char *)&usb_val, 1, ftdi->usb_read_timeout) != 1)
+    if (usb_control_msg(ftdi->usb_dev, 0xC0, 0x0C, 0, ftdi->index, (char *)pins, 1, ftdi->usb_read_timeout) != 1)
         ftdi_error_return(-1, "read pins failed");
 
-    *pins = (unsigned char)usb_val;
     return 0;
 }
 
+/**
+    Set latency timer
 
+    The FTDI chip keeps data in the internal buffer for a specific
+    amount of time if the buffer is not full yet to decrease
+    load on the usb bus.
+
+    \param ftdi pointer to ftdi_context
+    \param latency Value between 1 and 255
+
+    \retval  0: all fine
+    \retval -1: latency out of range
+    \retval -2: unable to set latency timer
+*/
 int ftdi_set_latency_timer(struct ftdi_context *ftdi, unsigned char latency)
 {
     unsigned short usb_val;
@@ -779,7 +995,15 @@ int ftdi_set_latency_timer(struct ftdi_context *ftdi, unsigned char latency)
     return 0;
 }
 
+/**
+    Get latency timer
 
+    \param ftdi pointer to ftdi_context
+    \param latency Pointer to store latency value in
+
+    \retval  0: all fine
+    \retval -1: unable to get latency timer
+*/
 int ftdi_get_latency_timer(struct ftdi_context *ftdi, unsigned char *latency)
 {
     unsigned short usb_val;
@@ -790,7 +1014,11 @@ int ftdi_get_latency_timer(struct ftdi_context *ftdi, unsigned char *latency)
     return 0;
 }
 
+/**
+    Init eeprom with default values.
 
+    \param eeprom Pointer to ftdi_eeprom
+*/
 void ftdi_eeprom_initdefaults(struct ftdi_eeprom *eeprom)
 {
     eeprom->vendor_id = 0x0403;
@@ -814,16 +1042,15 @@ void ftdi_eeprom_initdefaults(struct ftdi_eeprom *eeprom)
     eeprom->serial = NULL;
 }
 
+/**
+   Build binary output from ftdi_eeprom structure.
+   Output is suitable for ftdi_write_eeprom().
 
-/*
-    ftdi_eeprom_build
-    
-    Build binary output from ftdi_eeprom structure.
-    Output is suitable for ftdi_write_eeprom.
-    
-    Return codes:
-    positive value: used eeprom size
-    -1: eeprom size (128 bytes) exceeded by custom strings
+   \param eeprom Pointer to ftdi_eeprom
+   \param output Buffer of 128 bytes to store eeprom image to
+
+   \retval >0: used eeprom size
+   \retval -1: eeprom size (128 bytes) exceeded by custom strings
 */
 int ftdi_eeprom_build(struct ftdi_eeprom *eeprom, unsigned char *output)
 {
@@ -979,7 +1206,15 @@ int ftdi_eeprom_build(struct ftdi_eeprom *eeprom, unsigned char *output)
     return size_check;
 }
 
+/**
+    Read eeprom
 
+    \param ftdi pointer to ftdi_context
+    \param eeprom Pointer to store eeprom into
+
+    \retval  0: all fine
+    \retval -1: read failed
+*/
 int ftdi_read_eeprom(struct ftdi_context *ftdi, unsigned char *eeprom)
 {
     int i;
@@ -992,7 +1227,15 @@ int ftdi_read_eeprom(struct ftdi_context *ftdi, unsigned char *eeprom)
     return 0;
 }
 
+/**
+    Write eeprom
 
+    \param ftdi pointer to ftdi_context
+    \param eeprom Pointer to read eeprom from
+
+    \retval  0: all fine
+    \retval -1: read failed
+*/
 int ftdi_write_eeprom(struct ftdi_context *ftdi, unsigned char *eeprom)
 {
     unsigned short usb_val;
@@ -1008,7 +1251,14 @@ int ftdi_write_eeprom(struct ftdi_context *ftdi, unsigned char *eeprom)
     return 0;
 }
 
+/**
+    Erase eeprom
 
+    \param ftdi pointer to ftdi_context
+
+    \retval  0: all fine
+    \retval -1: erase failed
+*/
 int ftdi_erase_eeprom(struct ftdi_context *ftdi)
 {
     if (usb_control_msg(ftdi->usb_dev, 0x40, 0x92, 0, 0, NULL, 0, ftdi->usb_write_timeout) != 0)
@@ -1017,8 +1267,92 @@ int ftdi_erase_eeprom(struct ftdi_context *ftdi)
     return 0;
 }
 
+/**
+    Get string representation for last error code
 
+    \param ftdi pointer to ftdi_context
+
+    \retval Pointer to error string
+*/
 char *ftdi_get_error_string (struct ftdi_context *ftdi)
 {
     return ftdi->error_str;
 }
+
+/*
+    Flow control code by Lorenz Moesenlechner (lorenz@hcilab.org)
+    and Matthias Kranz  (matthias@hcilab.org)
+*/
+/**
+    Set flowcontrol for ftdi chip
+
+    \param ftdi pointer to ftdi_context
+    \param flowctrl flow control to use. should be 
+           SIO_DISABLE_FLOW_CTRL, SIO_RTS_CTS_HS, SIO_DTR_DSR_HS or SIO_XON_XOFF_HS   
+
+    \retval  0: all fine
+    \retval -1: set flow control failed
+*/
+int ftdi_setflowctrl(struct ftdi_context *ftdi, int flowctrl)
+{
+    if (usb_control_msg(ftdi->usb_dev, SIO_SET_FLOW_CTRL_REQUEST_TYPE,
+                        SIO_SET_FLOW_CTRL_REQUEST, 0, (flowctrl | ftdi->interface),
+                        NULL, 0, ftdi->usb_write_timeout) != 0)
+        ftdi_error_return(-1, "set flow control failed");
+
+    return 0;
+}
+
+/**
+    Set dtr line
+
+    \param ftdi pointer to ftdi_context
+    \param state state to set line to (1 or 0)
+
+    \retval  0: all fine
+    \retval -1: set dtr failed
+*/
+int ftdi_setdtr(struct ftdi_context *ftdi, int state)
+{
+    unsigned short usb_val;
+
+    if (state)
+        usb_val = SIO_SET_DTR_HIGH;
+    else
+        usb_val = SIO_SET_DTR_LOW;
+
+    if (usb_control_msg(ftdi->usb_dev, SIO_SET_MODEM_CTRL_REQUEST_TYPE,
+                        SIO_SET_MODEM_CTRL_REQUEST, usb_val, ftdi->interface,
+                        NULL, 0, ftdi->usb_write_timeout) != 0)
+        ftdi_error_return(-1, "set dtr failed");
+
+    return 0;
+}
+
+/**
+    Set rts line
+
+    \param ftdi pointer to ftdi_context
+    \param state state to set line to (1 or 0)
+
+    \retval  0: all fine
+    \retval -1 set rts failed
+*/
+int ftdi_setrts(struct ftdi_context *ftdi, int state)
+{
+    unsigned short usb_val;
+
+    if (state)
+        usb_val = SIO_SET_RTS_HIGH;
+    else
+        usb_val = SIO_SET_RTS_LOW;
+
+    if (usb_control_msg(ftdi->usb_dev, SIO_SET_MODEM_CTRL_REQUEST_TYPE,
+                        SIO_SET_MODEM_CTRL_REQUEST, usb_val, ftdi->interface,
+                        NULL, 0, ftdi->usb_write_timeout) != 0)
+        ftdi_error_return(-1, "set of rts failed");
+
+    return 0;
+}
+
+/* @} end of doxygen libftdi group */
